#!/usr/bin/nodejs

const NALseparator = new Buffer([0,0,0,1]); // NAL break

var args = process.argv.slice(2);

var options = {
	width : args[0],
	height: args[1],
	fps: args[2],
}

var clientsCount = 0;


///////////////////////////////////////////////////////////////////////////////
//
// UDP capture
//
///////////////////////////////////////////////////////////////////////////////
var dgram = require('dgram');
var server = dgram.createSocket('udp4');

server.on('listening', function () {
   	var address = server.address();
    console.log('UDP Server listening on ' + address.address + ":" + address.port);
});

server.on('message', function (data, remote) {
	//console.log(remote.address + ':' + remote.port);

	stream_transform(data);
});

server.bind(3001, "127.0.0.1");



///////////////////////////////////////////////////////////////////////////////
//
// WebSockets
//
///////////////////////////////////////////////////////////////////////////////

var wsServer = require('./wsserver.js');

wsServer.onConnected = function() {
	console.log("client connected");
}

function send_chunk(data) {
	//console.log(data);

	try {
		wsServer.send(Buffer.concat([NALseparator, data]));
	} catch (e) {}
}

wsServer.start(8080, options);



///////////////////////////////////////////////////////////////////////////////
//
// Controls
//
///////////////////////////////////////////////////////////////////////////////

setInterval(check_clients, 1000);

function check_clients() {

	// check current clients count
	if (wsServer.sockets) {
		clientsCount = Object.keys(wsServer.sockets).length
	}

	console.log("clientsCount " + clientsCount);

	if (clientsCount > 0) {
	} else {
	}
}


///////////////////////////////////////////////////////////////////////////////
//
// Stream transform
//
///////////////////////////////////////////////////////////////////////////////

var offset = 0;
var bodyOffset = 0;

var bufferSize  = 1024 * 1024 * 1  ; // 1Mb
var bufferFlush = Math.floor(bufferSize * 0.1); // 10% buffer size

var buffer = new Buffer(bufferSize); buffer.fill(0);
var separator = NALseparator;

function stream_transform(chunk) {
	if (offset + chunk.length > bufferSize - bufferFlush) {
		var minimalLength = bufferSize - bodyOffset + chunk.length;
		if(bufferSize < minimalLength) {
			//console.warn("Increasing buffer size to ", minimalLength);
			bufferSize = minimalLength;
		}
          
		var tmp = new Buffer(bufferSize);
		buffer.copy(tmp, 0, bodyOffset);
		buffer = tmp;
		offset = offset - bodyOffset;
		bodyOffset = 0;
	}

	chunk.copy(buffer, offset);

	var i, start, stop = offset + chunk.length;
	do {
		start = Math.max(bodyOffset ? bodyOffset : 0, offset - separator.length);
		i = buffer.slice(start, stop).indexOf(separator);

		if (i == -1) break;

		i += start;

		var img = buffer.slice(bodyOffset, i);
		send_chunk(img);

		bodyOffset = i + separator.length;
	} while(true);

	offset += chunk.length;
}
